"""
Compatibility package for xwlazy imports.

This package provides backward compatibility for packages that import
from xwlazy instead of exonware.xwlazy.

Company: eXonware.com
Author: Eng. Muhammad AlShehri
Email: connect@exonware.com
"""

# Re-export everything from the main package
from exonware.xwlazy import *  # noqa: F401, F403
