"""
Package Data - Immutable data structure for packages.

Company: eXonware.com
Author: Eng. Muhammad AlShehri
Email: connect@exonware.com

Generation Date: 15-Nov-2025

Re-export PackageData from defs.py for backward compatibility.
"""

# Re-export from defs.py
from ..defs import PackageData

__all__ = ['PackageData']
